/*
 * Copyright (c) 2007, 2008 Alexey Vatchenko <av@bsdua.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <sys/types.h>
#include <sys/wait.h>

#include <ctype.h>
#include <err.h>
#include <fcntl.h>
#include <limits.h>
#include <readpassphrase.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>

#include "cdma.h"
#include "dev.h"
#include "util.h"

#define MAX_PINCODE_SIZE		1024
#define MAX_PB_NUMBER			250

extern char *__progname;

static const char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
    "Aug", "Sep", "Oct", "Nov", "Dec"};
static const char *wdays[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};

static __dead void usage(void);

int	init_date(struct cdma_time *ct, time_t *t);
int	convert_date(const char *date, struct cdma_time *ct);
void	complete_cdma_time(struct cdma_time *ct);

int	cdma_init(struct cdma *ctx, const char *dev, const char *model);
void	cdma_final(struct cdma *ctx);
int	cdma_auth(struct cdma *ctx);

/*
 * Time parsing and processing
 */
int	cdma_time_show(struct cdma *ctx);
int	cdma_time_set(struct cdma *ctx);

/*
 * Pin code framework
 */
int	enter_pin(struct cdma *ctx);
int	enter_puk(struct cdma *ctx);
int	cdma_pin_show(struct cdma *ctx);
int	cdma_pin_enable(struct cdma *ctx, int toenable);
int	cdma_pin_change(struct cdma *ctx);

/*
 * Phone Book framework
 */
char *	validate_phone(const char *str);
char *	validate_name(const char *str);
void	output_pb(const char *idx, const char *phone, const char *name);
int	cdma_phonebook_show(struct cdma *ctx);
int	cdma_phonebook_add(struct cdma *ctx, u_int index, const char *phone,
	    const char *name);
int	cdma_phonebook_delete(struct cdma *ctx, u_int index);

/*
 * Mode framework
 */
int	cdma_mode_show(struct cdma *ctx);
int	cdma_mode_set(struct cdma *ctx, const char *mode);

/*
 * SMS framework
 */
void	output_sms(const char *index, struct cdma_sms *sms);
int	cdma_sms_show(struct cdma *ctx);
int	cdma_sms_check(struct cdma *ctx);
int	cdma_sms_send(struct cdma *ctx, const char *dst);
int	cdma_sms_read(struct cdma *ctx);
int	cdma_sms_delete(struct cdma *ctx, u_int index);
int	cdma_sms_save(struct cdma *ctx, const char *phone, const char *date);

static __dead void
usage(void)
{
	fprintf(stderr, "%s [-d device_model] [-f device] command\n"
	    "Commands:\n"
	    "\tlist\t\t- show all supported device models\n"
	    "\tdate\t\t- show date and time\n"
	    "\tdate set\t- set local time\n"
	    "\tmode\t\t- show current mode\n"
	    "\tmode hybrid\t- set hybrid mode\n"
	    "\tmode 1x\t\t- set 1x mode\n"
	    "\tmode ev-do\t- set EV-DO mode\n"
	    "\tpb\t\t- show address book\n"
	    "\tpb add <idx> <phone> <name>\t- add record with index <idx>\n"
	    "\tpb del <idx>\t- delete record by index <idx>\n"
	    "\tpin\t\t- show whether PIN code is enabled\n"
	    "\tpin enable\t- enable PIN code\n"
	    "\tpin disable\t- disable PIN code\n"
	    "\tpin change\t- change PIN code\n"
	    "\tsms\t\t- show stored messages\n"
	    "\tsms check\t- check for new SMS\n"
	    "\tsms send <phone>\t- send SMS (from stdin)\n"
	    "\tsms read\t- read new SMS (written to stdout)\n"
	    "\tsms del <idx>\t- delete stored SMS with index <idx>\n"
	    "\tsms save <phone> [date]\t- save SMS (from stdin) into SIM-card\n"
	    "\nDate format: [[[[[cc]yy]mm]dd]HH]MM[.SS]\n",
	    __progname);
	exit(1);
}

int
init_date(struct cdma_time *ct, time_t *t)
{
	time_t cur;
	struct tm *lt;

	cur = (t != NULL) ? *t : time(NULL);
	if (cur == -1)
		return (-1);

	lt = localtime(&cur);
	if (lt == NULL)
		return (-1);

	memset(ct, 0, sizeof(*ct));
	ct->year = lt->tm_year + 1900;
	ct->month = lt->tm_mon + 1;
	ct->day = lt->tm_mday;
	ct->hour = lt->tm_hour;
	ct->min = lt->tm_min;
	ct->sec = lt->tm_sec;
	strlcpy(ct->wday, wdays[lt->tm_wday % 7], sizeof(ct->wday));
	if (lt->tm_zone != NULL)
		strlcpy(ct->tz, lt->tm_zone, sizeof(ct->tz));
	return (0);
}

int
convert_date(const char *date, struct cdma_time *ct)
{
	char *s, *e, *p, *dot;
	int yr;

	/*
	 * Remove leading and trailing spaces
	 */
	s = (char *)date + strspn(date, " \t");
	if (*s == '\0')
		return (-1);	/* empty string */
	e = s + strcspn(s, " \t");
	/* check if it's not just a space inside of string */
	if (*e != '\0') {
		p = e + 1 + strspn(e + 1, " \t");
		if (*p != '\0')
			return (-1);
	}

	dot = NULL;
	for (p = s; p < e; p++) {
		if (*p == '.') {
			dot = p;	/* got sec separator */
			break;
		}
	}

	init_date(ct, NULL);

	if (dot != NULL) {
		if (e - dot - 1 != 2)
			return (-1);
		if (!isdigit(dot[1]) || !isdigit(dot[2]))
			return (-1);
		ct->sec = (dot[1] - '0') * 10 + (dot[2] - '0');
		e = dot;
	}

	for (p = s; p < e; p++) {
		if (!isdigit(*p))
			return (-1);
	}

	yr = 1;	/* if YY specified, ct->year must be reset before */
	switch (e - s) {
	case 12:
		yr = 0;
		ct->year = (s[0] - '0') * 1000 + (s[1] - '0') * 100;
		s += 2;
		/* FALLTHOUGH */
	case 10:
		if (yr) {
			ct->year /= 100;
			ct->year *= 100;
		}
		ct->year += (s[0] - '0') * 10 + (s[1] - '0');
		s += 2;
		/* FALLTHOUGH */
	case 8:
		ct->month = (s[0] - '0') * 10 + (s[1] - '0');
		if (ct->month < 1)
			ct->month = 1;
		else if (ct->month > 12)
			ct->month = 12;
		s += 2;
		/* FALLTHOUGH */
	case 6:
		ct->day = (s[0] - '0') * 10 + (s[1] - '0');
		s += 2;
		/* FALLTHOUGH */
	case 4:
		ct->hour = (s[0] - '0') * 10 + (s[1] - '0');
		s += 2;
		/* FALLTHOUGH */
	case 2:
		ct->min = (s[0] - '0') * 10 + (s[1] - '0');
		s += 2;
		break;
	default:
		return (-1);
	}

	return (0);
}

void
complete_cdma_time(struct cdma_time *ct)
{
	struct tm mkt;
	time_t t;

	memset(&mkt, 0, sizeof(mkt));
	mkt.tm_zone = "";
	mkt.tm_isdst = -1;
	mkt.tm_year = ct->year - 1900;
	mkt.tm_mon = ct->month - 1;
	mkt.tm_mday = ct->day;
	mkt.tm_hour = ct->hour;
	mkt.tm_min = ct->min;
	mkt.tm_sec = ct->sec;
	t = mktime(&mkt);
	if (t == -1 || init_date(ct, &t) == -1) {
		strlcpy(ct->wday, "???", sizeof(ct->wday));
		strlcpy(ct->tz, "???", sizeof(ct->tz));
	}
}

int
cdma_init(struct cdma *ctx, const char *dev, const char *model)
{
	ctx->buf = NULL;
	ctx->bufsz = 0;
	ctx->fd = open_tty(dev);
	if (ctx->fd != -1) {
		ctx->dev = (model != NULL) ? cdma_dev_id(model) :
		    cdma_dev_detect(ctx);
		if (ctx->dev == NULL) {
			close(ctx->fd);
			ctx->fd = -1;
		}
	}
	return ((ctx->fd == -1) ? -1 : 0);
}

void
cdma_final(struct cdma *ctx)
{
	if (ctx->buf != NULL)
		free(ctx->buf);
	close(ctx->fd);
}

int
cdma_auth(struct cdma *ctx)
{
	int flags, error;

	for (;;) {
		error = ctx->dev->pin_info(ctx, &flags);
		if (error != 0)
			return (-1);

		if ((flags & PIN_NEEDED) == 0)
			return (0);

		if ((flags & PIN_PUK) == 0)
			error = enter_pin(ctx);
		else
			error = enter_puk(ctx);
		if (error != 0)
			return (-1);
	}
	/* NOTREACHED */
}

int
cdma_time_show(struct cdma *ctx)
{
	struct cdma_time ct;
	struct tm *lt;
	time_t t;
	char *tz_name;
	int error;

	error = ctx->dev->get_time(ctx, &ct);
	if (error != 0)
		return (-1);

	/* Try to get local TZ abbr. */
	tz_name = NULL;
	t = time(NULL);
	if (t != -1) {
		lt = localtime(&t);
		if (lt != NULL)
			tz_name = lt->tm_zone;
	}

	if (tz_name == NULL)
		tz_name = "GMT";	/* XXX */

	printf("%3s %3s %2d %02d:%02d:%02d %s %4d\n", ct.wday,
	    months[(ct.month - 1) % 12], ct.day, ct.hour, ct.min, ct.sec,
	    tz_name, ct.year);
	return (0);
}

int
cdma_time_set(struct cdma *ctx)
{
	struct cdma_time ct;
	int error, status;
	char datetime[16]; /* YYYYMMDDHHMM.SS */
	pid_t pid;

	error = ctx->dev->get_time(ctx, &ct);
	if (error != 0)
		return (-1);

	pid = fork();
	if (pid == -1) {
		warn("can't set time");
		return (-1);
	} else if (pid == 0) {
		error = snprintf(datetime, sizeof(datetime),
		    "%04d%02d%02d%02d%02d.%02d",
		    ct.year, ct.month, ct.day, ct.hour, ct.min, ct.sec);
		if (error == -1 || (size_t)error >= sizeof(datetime))
			_exit(1);

		*(datetime + sizeof(datetime) - 1) = '\0';
		execl("/bin/date", "/bin/date", datetime, NULL);
		_exit(1);
	}

	pid = waitpid(pid, &status, 0);
	if (pid != -1 && WIFEXITED(status) &&
	    (WEXITSTATUS(status) != 0 && WEXITSTATUS(status) != 2))
		warnx("can't set time");

	return (0);
}

int
enter_pin(struct cdma *ctx)
{
	char pin[MAX_PINCODE_SIZE];

	if (readpassphrase("PIN code: ", pin, sizeof(pin), 0) == NULL) {
		warn("can't read PIN code");
		return (-1);
	}

	return ctx->dev->pin_enter(ctx, pin);
}

int
enter_puk(struct cdma *ctx)
{
	char puk[MAX_PINCODE_SIZE];
	char pin[MAX_PINCODE_SIZE], repin[MAX_PINCODE_SIZE];

	if (readpassphrase("PUK code: ", puk, sizeof(puk), 0) == NULL) {
		warnx("can't read PUK code");
		return (-1);
	}

	if (readpassphrase("Enter new PIN code: ",
	    pin, sizeof(pin), 0) == NULL) {
		warnx("can't read PIN code");
		return (-1);
	}

	if (readpassphrase("Re-enter new PIN code: ",
	    repin, sizeof(repin), 0) == NULL) {
		warnx("can't read PIN code");
		return (-1);
	}

	if (strcmp(pin, repin) != 0) {
		warnx("PIN codes don't match");
		return (0);	/* XXX - ??? */
	}

	return ctx->dev->puk_enter(ctx, puk, pin);
}

int
cdma_pin_show(struct cdma *ctx)
{
	int flags;
	int error;

	error = ctx->dev->pin_info(ctx, &flags);
	if (error == -1)
		return (-1);

	printf("PIN code is %s\n", ((flags & PIN_ENABLED) != 0) ? "enabled" :
	    "disabled");
	return (0);
}

int
cdma_pin_enable(struct cdma *ctx, int toenable)
{
	char buf[MAX_PINCODE_SIZE];

	if (readpassphrase("Enter PIN code to confirm: ",
	    buf, sizeof(buf), 0) == NULL) {
		warn("can't read PIN code");
		return (-1);
	}

	return ctx->dev->pin_enable(ctx, buf, toenable);
}

int
cdma_pin_change(struct cdma *ctx)
{
	char oldpin[MAX_PINCODE_SIZE], newpin[MAX_PINCODE_SIZE];
	char newpin2[MAX_PINCODE_SIZE];
	int flags, error;

	error = ctx->dev->pin_info(ctx, &flags);
	if (error == -1)
		return (-1);

	if ((flags & PIN_ENABLED) == 0) {
		warnx("PIN code must be enabled before changing");
		return (0);
	}

	if (readpassphrase("Old PIN code: ", oldpin, sizeof(oldpin),
	    0) == NULL || readpassphrase("New PIN code: ", newpin,
	    sizeof(newpin), 0) == NULL ||
	    readpassphrase("Re-enter new PIN code:", newpin2, sizeof(newpin2),
	    0) == NULL) {
		warn("can't read PIN codes");
		return (-1);
	}

	if (strcmp(newpin, newpin2) != 0) {
		warnx("Probably you mistyped new PIN code");
		return (-1);
	}

	return ctx->dev->pin_change(ctx, oldpin, newpin);
}

char *
validate_phone(const char *str)
{
	size_t i, len;

	len = strlen(str);
	if (len == 0)
		return (NULL);

	for (i = 0; i < len; i++) {
		if (!isdigit(str[i]) && !isalpha(str[i]) &&
		    str[i] != '+' && str[i] != '(' && str[i] != ')' &&
		    str[i] != ' ' && str[i] != '@')
			return (NULL);
	}
	return ((char *)str);
}

char *
validate_name(const char *str)
{
	size_t i, len;

	len = strlen(str);
	if (len == 0)
		return (NULL);

	for (i = 0; i < len; i++) {
		if (!isdigit(str[i]) && !isalpha(str[i]) &&
		    str[i] != '+' && str[i] != '(' && str[i] != ')' &&
		    str[i] != ' ' && str[i] != '@' && str[i] != '.' &&
		    str[i] != '<' && str[i] != '>')
			return (NULL);
	}
	return ((char *)str);
}

void
output_pb(const char *idx, const char *phone, const char *name)
{
	printf("%-3s  %-15s\t%s\n", idx, phone, name);
}

int
cdma_phonebook_show(struct cdma *ctx)
{
	return ctx->dev->pb_enum(ctx, output_pb);
}

int
cdma_phonebook_add(struct cdma *ctx, u_int index, const char *phone,
    const char *name)
{
	size_t plen;
	int plim, nlim;

	plim = ctx->dev->get_limit(ctx, CDMA_LIMIT_PHONE);
	nlim = ctx->dev->get_limit(ctx, CDMA_LIMIT_CONTACT);

	if ((plen = strlen(phone)) < 1) {
		warnx("phone number is not specified");
		return (-1);
	} else if (plim != 0 && plen > (size_t)plim) {
		warnx("phone number is too long. Must be %d symbols long.",
		    plim);
		return (-1);
	} else if (validate_phone(phone) == NULL) {
		warnx("invalid phone number");
		return (-1);
	} else if (nlim != 0 && strlen(name) > (size_t)nlim) {
		warnx("contact name is too long. Must be %d symbols long.",
		    nlim);
		return (-1);
	} else if (validate_name(name) == NULL) {
		warnx("invalid contact name");
		return (-1);
	}
	return ctx->dev->pb_add(ctx, index, phone, name);
}

int
cdma_phonebook_delete(struct cdma *ctx, u_int index)
{
	return ctx->dev->pb_delete(ctx, index);
}

int
cdma_mode_show(struct cdma *ctx)
{
	return ctx->dev->get_mode(ctx);
}

int
cdma_mode_set(struct cdma *ctx, const char *mode)
{
	int val;

	if (strcasecmp(mode, "hybrid") == 0)
		val = 0;
	else if (strcasecmp(mode, "1x") == 0)
		val = 1;
	else if (strcasecmp(mode, "ev-do") == 0)
		val = 2;
	else {
		warnx("invalid mode");
		return (-1);
	}

	return ctx->dev->set_mode(ctx, val);
}

void
output_sms(const char *index, struct cdma_sms *sms)
{
	complete_cdma_time(&sms->when);

	printf("--[%3s]----\n", index);
	printf("From: %s\n", sms->from);
	printf("Date: %s, %d %s %d %02d:%02d:%02d %s\n", sms->when.wday,
	    sms->when.day, months[(sms->when.month - 1) % 12], sms->when.year,
	    sms->when.hour, sms->when.min, sms->when.sec, sms->when.tz);
	printf("\n%s\n", sms->msg);
}

int
cdma_sms_show(struct cdma *ctx)
{
	return ctx->dev->sms_enum(ctx, output_sms);
}

int
cdma_sms_check(struct cdma *ctx)
{
	u_int n_sms;
	int error;

	error = ctx->dev->sms_check(ctx, &n_sms);
	if (error != 0)
		return (-1);

	printf("You have %u unread SMS\n", n_sms);
	return (0);
}

int
cdma_sms_send(struct cdma *ctx, const char *dst)
{
	size_t sz, i;
	int error;

	if (validate_phone(dst) == NULL) {
		warnx("invalid phone number");
		return (-1);
	}

	error = read_stdin(&ctx->buf, &ctx->bufsz, &sz);
	if (error != 0)
		return (-1);

	/*
	 * Remove last \r and \n symbols if they are not the only symbols.
	 */
	if (sz == 0) {
		warnx("empty message, nothing to send");
		return (-1);
	}
	for (i = sz; i > 0; i--) {
		if (ctx->buf[i - 1] != '\r' && ctx->buf[i - 1] != '\n')
			break;
	}

	if (i > 0)
		sz = i;

	return ctx->dev->sms_send(ctx, dst, ctx->buf, sz);
}

int
cdma_sms_read(struct cdma *ctx)
{
	struct cdma_sms sms;
	int error;

	error = ctx->dev->sms_read(ctx, &sms);
	if (error != 1)
		return (error);

	complete_cdma_time(&sms.when);

	printf("From: %s\n", sms.from);
	printf("Date: %s, %d %s %d %02d:%02d:%02d %s\n", sms.when.wday,
	    sms.when.day, months[(sms.when.month - 1) % 12], sms.when.year,
	    sms.when.hour, sms.when.min, sms.when.sec, sms.when.tz);
	printf("\n%s\n", sms.msg);
	return (0);
}

int
cdma_sms_delete(struct cdma *ctx, u_int index)
{
	return ctx->dev->sms_delete(ctx, index);
}

int
cdma_sms_save(struct cdma *ctx, const char *phone, const char *date)
{
	struct cdma_time ct;
	char *p;
	size_t i, rdsz;
	int error;

	if (validate_phone(phone) == NULL)
		return (-1);

	error = read_stdin(&ctx->buf, &ctx->bufsz, &rdsz);
	if (error != 0)
		return (error);

	/*
	 * Remove last \r and \n symbols if they are not the only symbols.
	 */
	if (rdsz > 0) {
		for (i = rdsz; i > 0; i--) {
			if (ctx->buf[i - 1] != '\r' && ctx->buf[i - 1] != '\n')
				break;
		}

		if (i > 0)
			ctx->buf[i] = '\0';
	}

	if (rdsz == 0) {
		warnx("empty message, nothing to save");
		return (-1);
	}

	/*
	 * Remove all \r symbols in msg because it's AT commands separator.
	 */
	p = ctx->buf;
	while (*p != '\0') {
		if (*p == '\r')
			*p = ' ';
		p++;
	}

	if (date != NULL) {
		if (convert_date(date, &ct) == -1) {
			warnx("invalid date specified");
			return (-1);
		}
	} else {
		if (init_date(&ct, NULL) == -1) {
			warnx("error while initializing time");
			return (-1);
		}
	}

	return ctx->dev->sms_save(ctx, phone, &ct, ctx->buf);
}

int
main(int argc, char * const *argv)
{
	struct cdma c;
	char *devfile, *devmodel, *date;
	const char *errstr;
	u_int idx;
	int ch, error;

	devfile = devmodel = NULL;

	while ((ch = getopt(argc, argv, "d:f:h")) != -1) {
		switch (ch) {
		case 'd':
			devmodel = optarg;
			break;
		case 'f':
			devfile = optarg;
			break;
		case 'h':
		default:
			usage();
			/* NOTREACHED */
		}
	}

	argc -= optind;
	argv += optind;

	if (argc < 1) {
		/* XXX - interactive mode */
		usage();
		/* NOTREACHED */
	}

	if (strcmp(*argv, "list") == 0) {
		cdma_dev_list();
		return (0);
	}

	error = cdma_init(&c, devfile, devmodel);
	if (error == -1)
		return (1);

	error = cdma_auth(&c);
	if (error == -1)
		return (2);

	if (strcmp(*argv, "date") == 0) {
		if (argc == 1)
			cdma_time_show(&c);
		else if (strcmp(argv[1], "set") == 0)
			cdma_time_set(&c);
		else
			warnx("invalid command");
	} else if (strcmp(*argv, "mode") == 0) {
		if (argc == 1)
			cdma_mode_show(&c);
		else if (argc > 1) {
			if (cdma_mode_set(&c, argv[1]) == 0)
				cdma_mode_show(&c);
		} else
			warnx("invalid command");
	} else if (strcmp(*argv, "pb") == 0) {
		if (argc == 1)
			cdma_phonebook_show(&c);
		else if (argc > 4 && strcmp(argv[1], "add") == 0) {
			idx = strtonum(argv[2], 1, MAX_PB_NUMBER, &errstr);
			if (errstr != NULL) {
				warnx("index must be numeric value in range "
				    "1-%u", MAX_PB_NUMBER);
			} else
				cdma_phonebook_add(&c, idx, argv[3], argv[4]);
		} else if (argc > 2 && strcmp(argv[1], "del") == 0) {
			idx = strtonum(argv[2], 1, MAX_PB_NUMBER, &errstr);
			if (errstr != NULL) {
				warnx("index must be numeric value in range "
				    "1-%u", MAX_PB_NUMBER);
			} else
				cdma_phonebook_delete(&c, idx);
		} else
			warnx("invalid command");
	} else if (strcmp(*argv, "pin") == 0) {
		if (argc == 1)
			cdma_pin_show(&c);
		else if (argc > 1 && strcmp(argv[1], "enable") == 0) {
			if (cdma_pin_enable(&c, 1) == 0)
				cdma_pin_show(&c);
		} else if (argc > 1 && strcmp(argv[1], "disable") == 0) {
			if (cdma_pin_enable(&c, 0) == 0)
				cdma_pin_show(&c);
		} else if (argc > 1 && strcmp(argv[1], "change") == 0)
			cdma_pin_change(&c);
		else
			warnx("invalid command");
	} else if (strcmp(*argv, "sms") == 0) {
		if (argc == 1)
			cdma_sms_show(&c);
		else if (argc > 1 && strcmp(argv[1], "check") == 0)
			cdma_sms_check(&c);
		else if (argc > 2 && strcmp(argv[1], "del") == 0) {
			idx = strtonum(argv[2], 1, UINT_MAX, &errstr);
			if (errstr != NULL)
				warnx("index must be numeric value");
			else
				cdma_sms_delete(&c, idx);
		} else if (argc > 2 && strcmp(argv[1], "send") == 0)
			cdma_sms_send(&c, argv[2]);
		else if (argc > 1 && strcmp(argv[1], "read") == 0)
			cdma_sms_read(&c);
		else if (argc > 2 && strcmp(argv[1], "save") == 0) {
			date = (argc > 3) ? argv[3] : NULL;
			cdma_sms_save(&c, argv[2], date);
		} else
			warnx("invalid command");
	} else
		warnx("invalid command");

	cdma_final(&c);
	return (0);
}
