/*
 * Copyright (c) 2007 Alexey Vatchenko <av@bsdua.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <sys/types.h>

#include <err.h>
#include <stdio.h>
#include <string.h>

#include "dev.h"
#include "util.h"

#include "cmotech_ccu550.h"
#include "cmotech_ccu650u.h"

/* Macro for device definition */
#define DEFINE_CDMA_DEVICE(dev_id, dev_name, dev)	{ \
	dev_id, dev_name, dev##_getlimit, dev##_gettime, dev##_pininfo, \
	dev##_pin_enable, dev##_pin_change, dev##_pin_enter, dev##_puk_enter, \
	dev##_getmode, dev##_setmode, dev##_sms_send, dev##_sms_read, \
	dev##_sms_check, dev##_sms_enum, dev##_sms_delete, dev##_sms_save, \
	dev##_pb_enum, dev##_pb_add, dev##_pb_delete}

struct cdma_dev cdma_device[] = {
	DEFINE_CDMA_DEVICE("CCUTSU02", "C-motech CCU-550", cmotech_ccu550),
	DEFINE_CDMA_DEVICE("CCU65TSU00", "C-motech CCU-650U", cmotech_ccu650u)
};

void
cdma_dev_list(void)
{
	u_int i;

	for (i = 0; i < sizeof(cdma_device) / sizeof(*cdma_device); i++)
		printf("%s\t - %s\n", cdma_device[i].id, cdma_device[i].name);
}

struct cdma_dev *
cdma_dev_id(const char *id)
{
	u_int i;

	for (i = 0; i < sizeof(cdma_device) / sizeof(*cdma_device); i++)
		if (strcasecmp(cdma_device[i].id, id) == 0)
			return (cdma_device + i);
	warnx("modem is not supported");
	return (NULL);
}

/*
 * There are two (known for me) AT commands for determining modem model.
 * Use second one if first one is not supported by device.
 */
struct cdma_dev *
cdma_dev_detect(struct cdma *ctx)
{
	struct cdma_dev *rv;
	char *cmd[] = {"AT$$SWVER?\r\n", "$$SWVER", "AT+GMR\r\n", "+GMR", NULL};
	char **p, *ln, *nxt, *s;
	size_t sz, sz2;
	u_int i;
	int status, error;

	rv = NULL;
	for (p = (char **)cmd; *p != NULL && rv == NULL; p += 2) {
		error = send_cmd(ctx->fd, *p);
		if (error != 0)
			continue;

		error = recv_answer(ctx->fd, &ctx->buf, &ctx->bufsz, &sz,
		    &status);
		if (error != 0)
			continue;
		if (status != ANSWER_OK)
			continue;

		ln = ctx->buf;
		sz2 = sz;
		while (ln != NULL && rv == NULL) {
			ln = get_tagged_line(ln, sz2, *(p + 1), &nxt);
			if (ln == NULL)
				break;
			sz2 = sz - (ln - ctx->buf);

			s = get_answer_array(ln, sz2, 1, *(p + 1));
			if (s == NULL)
				break;

			/*
			 * We use strstr() for checking for modem model because
			 * format of output of this command is not strict.
			 */
			for (i = 0;
			    i < sizeof(cdma_device) / sizeof(*cdma_device);
			    i++) {
				if (strstr(s, cdma_device[i].id) != NULL) {
					rv = cdma_device + i;
					break;
				}
			}

			ln = nxt;
		}
	}

	if (rv == NULL)
		warnx("can't determine modem's model");
	return (rv);
}
